/*  Copyright (C) 2014 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "libknot/attribute.h"
#include "libknot/errcode.h"
#include "libknot/rrtype/aaaa.h"
#include "contrib/sockaddr.h"

_public_
int knot_a_addr(const knot_rdataset_t *rrs, size_t pos, struct sockaddr_in *dst)
{
	KNOT_RDATASET_CHECK(rrs, pos, return KNOT_EINVAL);
	knot_rdata_t *rr = knot_rdataset_at(rrs, pos);
	return sockaddr_set_raw((struct sockaddr_storage *)dst, AF_INET,
	                        knot_rdata_data(rr), knot_rdata_rdlen(rr));
}

_public_
int knot_aaaa_addr(const knot_rdataset_t *rrs, size_t pos, struct sockaddr_in6 *dst)
{
	KNOT_RDATASET_CHECK(rrs, pos, return KNOT_EINVAL);
	knot_rdata_t *rr = knot_rdataset_at(rrs, pos);
	return sockaddr_set_raw((struct sockaddr_storage *)dst, AF_INET6,
	                        knot_rdata_data(rr), knot_rdata_rdlen(rr));
}
